COMMENT ^
This module does not call any external procedures or use any external variables directly.
It contains two procedures:

	MoveDockedWindow- this moves the docked window witht he mina window
	DockWindow 		- This procedure docks a popup window to the main window if the windows are
					  no more than 10 pixels apart

; ###################################################################################################^
	.386                   ; minimum processor needed for 32 bit
	.model flat, stdcall   ; FLAT memory model & STDCALL calling
	option casemap :none   ; set code to case sensitive
; ###################################################################################################   
	include \masm32\include\windows.inc
	include \masm32\include\user32.inc
	includelib \masm32\lib\user32.lib
; ###################################################################################################
.data
; ###################################################################################################
.code

start:
; ###################################################################################################

DockWindow proc public uses edi ecx, ptrSecondWindowPos:Dword,DockState : Byte, DockedEdge : Byte,IsPopupActive: Byte,hParentWnd:Dword
; ********************************************************************************************************************
; PURPOSE: This procedure docks a window or dialog box to an edge of the main window
;           
; USE: 	This procedure is called by the following code which should be placed in the callback procedure
;		for the PopUp window or dialog: 
;
;		.elseif uMsg == WM_WINDOWPOSCHANGING
;			invoke DockWindow, lParam	;lparam points to a WindowPos structure
;
; ON ENTRY: ptrSecondWindowPos is an lParam pointer to a WindowPos structure for the second window
;			DockState indicates whether or not the window is docked - Values (T)rue or (F)alse
;			DockedEdge indicates which edge of the main window, if any, is docked - Values L, R, T, B
;			IsPopupActive indicates whether or not the popup window is has the focus - Values (T)rue or (F)alse
;			hParentWindow is the handle of the main window that the second window will dock to
;			
; USES:     hParentWnd - the handle of the main window
;           
; NOTE:     Adapted from a program written by Donkey for GoASM
;             
; ********************************************************************************************************************

	LOCAL MainWinRect : RECT
	LOCAL DokState : BYTE
	LOCAL DokEdge : BYTE
	
	mov al, DockState
	mov DokState,al
	mov al,DockedEdge
	mov DokEdge,al
	
	invoke GetWindowRect, hParentWnd, ADDR MainWinRect    ;get main window co-ords into MainWinRect: left,top,right,bottom
	mov edi,ptrSecondWindowPos			;edi holds address of a WindowPos Structure for the 2nd window
										;compare the tops to see if they are close enough
	mov eax,[edi+WINDOWPOS.y]			;Get top edge of second window
	sub eax,[MainWinRect.top]			;Sub top edge of main window
	jns @F                              ;if result in eax is negative
	neg eax                    		    ;then make eax positive
@@:	
	cmp eax,10                          ;are the tops of each window more than 10 pixels apart?	
	jg CheckBottom						;Popup window is not docked at the side of the main window
										;so check if it is docked at top/bottom
	mov ecx,[MainWinRect.right]			;Get right edge of main window
	sub ecx,[edi+WINDOWPOS.x]			;Sub left edge of second window
	jns @F								;if result in eax is negative
	neg ecx                             ;then make eax positive
@@:
	cmp ecx,10							;are the sides greater than 10 pixels apart on the right side of main window?
	jg CheckLeft						;YES - Check left side
										;NO - Dock the window
	mov eax,[MainWinRect.right]			;Get the right co-ord of the main window
	mov [edi+WINDOWPOS.x],eax			;Store it in the left edge value of WindowPos for secondary window
	mov eax,[MainWinRect.top]			;Get the top co-ord of the main window
	mov [edi+WINDOWPOS.y],eax			;Store it in the top edge value of WindowPos for secondary window
	mov al,'R'							;Second window is docked to (R)ight of main window
	mov DokEdge,al
	mov DokState, 'T'					;Set Is2Docked flag to true
	jmp EndDocking						;exit
	
CheckLeft:		
	mov eax,[edi+WINDOWPOS.x] 			;Get left edge of secondary window
	add eax,[edi+WINDOWPOS.lx]			;add width of secondary window to this value (=right edge)
	mov ecx,[MainWinRect.left]			;Get left co-ord of main window
	sub ecx,eax							;subtract left co-ordinates of each window
	jns @F								;if result in eax is negative
	neg ecx                             ;then make eax positive
@@:
	cmp ecx,10							;are the sides greater than 10 pixels apart on the left side of main window?
	jg  CheckBottom						;if not, check for docking at bottom of main window
										;else Dock the window
	mov eax,[MainWinRect.left]			;Get left co-ord of main window
	sub eax,[edi+WINDOWPOS.lx]			;sub width of secondary window from this value  (=right edge)
	mov [edi+WINDOWPOS.x],eax			;set left co-ord of second window
	mov eax,[MainWinRect.top]			;get top co-ord of main window
	mov [edi+WINDOWPOS.y],eax			;set top co-ord of second window
	mov al,'L'							;Second window is docked to (L)eft of main window
	mov DokEdge,al					;Store docked edge of main window
	mov DokState, 'T'					;Set Is2Docked flag to true
	jmp EndDocking						;exit this procedure

CheckBottom:	
		;are left edges of the windows within 10 pixels of each other?	
	mov ecx,[MainWinRect.left]			;Get left edge of main window
	sub ecx,[edi+WINDOWPOS.x]			;Sub left edge of second window
	jns @F								;if result in ecx is negative
	neg ecx                             ;then make ecx positive
@@:
	cmp ecx,10							;are the left edges greater than 10 pixels apart?
	jg NotDocked						;Yes - end docking procedure	
	
	;is top of second window within 10 pixels of lower edge of main window?
	mov eax,[edi+WINDOWPOS.y]			;Get top edge of second window
	sub eax,[MainWinRect.bottom]		;Sub bottom edge of main window
	jns @F                              ;if result in eax is negative
	neg eax                    		    ;then make eax positive
@@:	
	cmp eax,10                          ;is top of 2nd window more than 10 pixels from bottom of main window?
	jg CheckMainTop                     ;If not, check edges at top of main window
										;YES - Dock the window at bottom of main window
	mov eax,[MainWinRect.bottom]		;Get the bottom co-ord of the main window
	mov [edi+WINDOWPOS.y],eax			;Store it in the top edge value of WindowPos for secondary window
	mov eax,[MainWinRect.left]			;Get the left co-ord of the main window
	mov [edi+WINDOWPOS.x],eax			;Store it in the left edge value of WindowPos for secondary window
	mov al,'B'							;Second window is docked to (B)ottom of main window
	mov DokEdge,al					;Store docked edge of main window
	mov DokState, 'T'					;Set Is2Docked flag to true
	jmp EndDocking						;exit

CheckMainTop:
	;is bottom edge of second window within 10 pixels of top edge of main window?
	mov eax,[edi+WINDOWPOS.y] 			;Get top edge of secondary window
	add eax,[edi+WINDOWPOS.cy]			;add height of secondary window to this value (=bottom edge)
	mov ecx,[MainWinRect.top]			;Get top co-ord of main window
	sub ecx,eax							;subtract co-ordinates of each window
	jns @F								;if result in eax is negative
	neg ecx                             ;then make ecx positive
@@:
	cmp ecx,10                          ;are the edges of each window more than 10 pixels apart?	
	jg EndDocking                       ;If so, jump to EndDocking
										;else Dock the window at the top of the main window
	mov eax,[MainWinRect.left]			;Get left co-ord of main window	
	mov [edi+WINDOWPOS.x],eax			;set left co-ord of second window
	
	mov eax,[MainWinRect.top]			;get top co-ord of main window
	sub eax,[edi+WINDOWPOS.cy]			;sub height of second window
	mov [edi+WINDOWPOS.y],eax			;set top co-ord of second window
	mov al,'T'							;Second window is docked to (B)ottom of main window
	mov DokEdge,al					;Store docked edge of main window
	mov DokState, 'T'					;Set Is2Docked flag to true
	jmp EndDocking
NotDocked:
	mov al,IsPopupActive				;is the popup window active?
	cmp al, TRUE					
	jne EndDocking						;No - end procedure
	mov DokState, 'F'					;Yes - set the Is2Docked flag to false
EndDocking:
	mov al, DokState					;return DockState in al
	mov ah, DokEdge					;return DockedEdge in ah
	ret
DockWindow endp

; ###################################################################################################

MoveDockedWindow proc public uses edi, ptrMainWindowPos:Dword, DockState : Byte, DockedEdge : Byte,hDockedWnd:DWORD
; ********************************************************************************************************************
; PURPOSE: 	This procedure moves a docked popup window or docked dialog box with the main window
;			when the main window is moved
;
; USE: 	This procedure is called by the following code which should be placed in the callback procedure
;		for the Main window:
;
;		.elseif uMsg == WM_WINDOWPOSCHANGED
;		invoke MoveDockedWindow,lParam
;
; ON ENTRY: ptrMainWindowPos is the lParam pointer to a WindowPos structure for the poopup window/dialog
;			
; USES:     hDockedWnd - the handle of the popup window or dialog box
;           
; NOTE:     
;             
; ********************************************************************************************************************

	LOCAL SecondWinRect : RECT
	LOCAL SecondWinWidth : DWORD
	LOCAL SecondWinHeight : DWORD
	LOCAL xPos,yPos : DWORD
	
		mov al,DockState					;If the popup window is not docked...
		cmp al, 'F'
		je EndPosChange						;...then exit this procedure
		
		invoke GetWindowRect, hDockedWnd, ADDR SecondWinRect    ;get 2nd window co-ords in SecondWinRect: left,top,right,bottom
		mov eax,SecondWinRect.right
		sub eax,SecondWinRect.left			;calculate width of second window
		mov SecondWinWidth,eax				;store width of second window
		mov eax,SecondWinRect.bottom
		sub eax,SecondWinRect.top			;calculate height of second window
		mov SecondWinHeight,eax				;store height of second window
		
		mov edi,ptrMainWindowPos ;ptrMainWindowPos holds a WinPos structure for the new position of the main window
		mov al,DockedEdge
		cmp al,'L'
		jne NotDLeft
			;Move popup window so that it keeps against the left edge of main window here	
		mov eax,[edi+WINDOWPOS.x]			;Get position of left edge of main window
		sub eax,SecondWinWidth
		mov xPos,eax						;xPos = MainLeft-SecondWidth
		mov eax,[edi+WINDOWPOS.y]			;Get top edge of main window
		mov yPos,eax						;yPos = value of top edge of main window
	
		invoke MoveWindow,hDockedWnd,xPos,yPos,SecondWinWidth,SecondWinHeight,TRUE
		jmp EndPosChange	
NotDLeft:
		cmp al,'R'
		jne NotDRight
		;Move popup window so that it keeps against the right edge of main window here
		mov eax,[edi+WINDOWPOS.x]			;Get position of left edge of main window
		add eax,[edi+WINDOWPOS.lx]			;add width of the main window
		mov xPos,eax
		mov eax,[edi+WINDOWPOS.y]			;Get top edge of main window
		mov yPos,eax						;yPos = value of top edge of main window			
		invoke MoveWindow,hDockedWnd,xPos,yPos,SecondWinWidth,SecondWinHeight,TRUE
		jmp EndPosChange
NotDRight:
		cmp al,'T'
		jne NotDTop
		;Move popup window so that it keeps against the top edge of main window here
		mov eax,[edi+WINDOWPOS.y]			;Get top edge of main window
		sub eax, SecondWinHeight			;Subtract height of popup window
		mov yPos,eax						;yPos = Top of main window - height of popup window
		mov eax,[edi+WINDOWPOS.x]			;Get position of left edge of main window
		mov xPos,eax
		invoke MoveWindow,hDockedWnd,xPos,yPos,SecondWinWidth,SecondWinHeight,TRUE 
		jmp EndPosChange
NotDTop:
		;Move popup window so that it keeps against the bottom edge of main window here
		mov eax,[edi+WINDOWPOS.y]			;Get top edge of main window
		add eax,[edi+WINDOWPOS.cy]			;Add height of main window
		mov yPos,eax						;= y position of popup window
		mov eax,[edi+WINDOWPOS.x]			;Get position of left edge of main window
		mov xPos,eax		
		invoke MoveWindow,hDockedWnd,xPos,yPos,SecondWinWidth,SecondWinHeight,TRUE 	
EndPosChange:	
	ret
MoveDockedWindow endp



end