COMMENT ^
This module uses two external procedures, MoveDockedWindow and DockWindow. These are stored in DockCode.asm
This module also declares the handle for the popup window to be external as this handle needs to be
accessed in the main module, DockWindow.asm.

; ###################################################################################################^
	.386					; minimum processor needed for 32 bit
	.model flat, stdcall	; FLAT memory model & STDCALL calling
	option casemap :none	; set code to case sensitive
	option proc :private	;make procedures in this module private to this module
							;this may be overidden in individual procedures
; ###################################################################################################
	include \masm32\include\windows.inc
	include \masm32\include\user32.inc
	includelib \masm32\lib\user32.lib
; ###################################################################################################
	CreateMyPopWin PROTO :DWORD, :DWord						;These procedures are defined in this module
	WndProc4MyPopup PROTO  :DWORD,: DWORD,: DWORD,: DWORD
	MoveDockedWin PROTO :Dword,:Dword
; ###################################################################################################
	MoveDockedWindow PROTO :Dword, :Byte, :Byte, :DWORD 	;This procedure is in the module DockCode.asm
	DockWindow PROTO :Dword, :Byte, :Byte, :Byte, :Dword	;This procedure is in the module DockCode.asm
	EXTERNDEF hPopupWnd2 :DWORD								;This variable needs to be accessed by the main
															;module, DockWindow.asm
; ###################################################################################################
.data
	szPopUpWinCaption   db "Popup Window",0
	szPopUpClassName    db "PopupWindowClass",0
	hPopupWnd2			dd 0
	hParentWindow		dd 0
	
	DockedEdge2 		db 'R'	;byte indicates which edge of main window is docked to the 2nd window
								;Values: L, R, T, B
	Is2Docked			db 'T'	;Assume second window is docked to begin - set False if desired
	Is2Active 			db 'F'	;Assume second window is not active at start
; ################################################################################################### 	
.code

start:
; ###################################################################################################

CreateMyPopWin Proc public  WinMainProcInst:DWORD, hParentWin : DWORD  
    ; -------------------------------------------------------------------------------
    ; Creates and registers a new popup window 
    ;
    ; Called by:				
    ;
    ; Uses globals:         szPopUpClassName
    ;                       szPopUpWinCaption
    ;                       hPopupWnd2
    ; Parameters:           PopX - x co-ordinate of popup
    ;                       PopY - y co-ordinate of popup
    ;                       PopWwd - Width of popup window in screen coordinates
    ;                       PopWht - Height of popup window in screen coordinates
    ;                       WinMainProcInst - HInst parameter of WinMain proc
    ; ------------------------------------------------------------------------------- 

        LOCAL pwc   	:WNDCLASSEX
        LOCAL PopWwd    :DWORD  ;Pop up window parameters
        LOCAL PopWht    :DWORD
        LOCAL PopX      :DWORD
        LOCAL PopY      :DWORD
        
		mov  PopWwd,    200     ;Set popup width
        mov  PopWht,    200		;Set popup Height
        mov  PopX,      400		;Set popup X Position
        mov  PopY,      200		;Set popup Y Position
        
        ; -------- Fill Popup WNDCLASSEX structure with required variables ----------

        mov  pwc.cbSize,sizeof WNDCLASSEX
        mov  pwc.style,CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW or CS_NOCLOSE
        mov  pwc.lpfnWndProc,offset WndProc4MyPopup      	;address of WndProc4MyPopup
        mov  pwc.cbClsExtra,NULL
        mov  pwc.cbWndExtra,NULL
        push WinMainProcInst
        pop  pwc.hInstance
        mov  pwc.hbrBackground,COLOR_BTNFACE+1     			;use system colors
        mov  pwc.lpszMenuName, NULL
        mov  pwc.lpszClassName,offset szPopUpClassName  	;popup window class name
        invoke LoadIcon,WinMainProcInst,500                 ;icon ID - resource icon
        mov  pwc.hIcon,eax
        invoke LoadCursor,NULL,IDC_ARROW         			;use system cursor
        mov  pwc.hCursor,eax
        mov  pwc.hIconSm,0

        invoke RegisterClassEx, ADDR pwc            		;register the popup window class

        ; --------Create and register Popup window ----------

        invoke CreateWindowEx,WS_EX_LEFT, ADDR szPopUpClassName, ADDR szPopUpWinCaption,        
                              WS_POPUP or WS_BORDER or WS_CAPTION,                            
                              PopX,PopY,PopWwd,PopWht, hParentWin,NULL, WinMainProcInst,NULL
        mov  hPopupWnd2,eax
        mov eax,hParentWin
        mov hParentWindow,eax
        mov eax,0
        ret
CreateMyPopWin endp
; ###################################################################################################

WndProc4MyPopup proc hWin: DWORD, uMsg: DWORD, wParam: DWORD, lParam: DWORD
;this is the callback procedure for popup window 2. It contains code to respond to messages
;that are sent to that window.

	.if uMsg == WM_ACTIVATE
		mov al, FALSE		;Assume second window is not active
		mov Is2Active,al
		mov eax, wParam		;Loword of wParam indicates whether window has been activated or deactivated
		and	eax,0FFFFh		;Get low word of wParam in eax 
		cmp eax,WA_INACTIVE	;is the popup window activate?
		je NotActive		;No - take no further action
		mov al,TRUE			;Yes - set Is2Active to TRUE
		mov Is2Active, al
NotActive:

    .elseif uMsg == WM_WINDOWPOSCHANGING
    	;lparam points to a WindowPos structure when this message is sent
    	;this structure holds details about the position of the popup window and its size
    	;when this message is used
    	
		invoke DockWindow,lParam,Is2Docked,DockedEdge2,Is2Active,hParentWindow	;code to dock at side of main window
		mov Is2Docked,al	;store details of whether the popup window is docked or not
		mov DockedEdge2,ah	;store details of which edge of the main window is docked

    .endif
    invoke DefWindowProc,hWin,uMsg,wParam,lParam
    ret
WndProc4MyPopup endp

; ###################################################################################################

MoveDockedWin proc public lParam,hParentWnd
; This proceduer is called by the main module, DockWindow.asm. The main module passes the value of lParam to
; this procedure (lParam holds the address of a WINPOS structure for the main window.)
; The invoke instruction uses the passed value of lParam for the main window and the variables defined in this
; module to call MoveDockedWindow in DockCode.asm

	invoke MoveDockedWindow,lParam,Is2Docked,DockedEdge2,hPopupWnd2
	ret
MoveDockedWin endp
	
end